@@ -27,6 +27,7 @@ import scipy.optimize as spo
 import tensorflow as tf
 import tensorflow_probability as tfp
 
+from .. import logging
 from ..space import Box, DiscreteSearchSpace, SearchSpace, SearchSpaceType, TaggedProductSearchSpace
 from ..types import TensorType
 from .interface import AcquisitionFunction
@@ -161,7 +162,7 @@ def generate_continuous_optimizer(
     num_initial_samples: int = NUM_SAMPLES_MIN,
     num_optimization_runs: int = 10,
     num_recovery_runs: int = 10,
-    optimizer_args: dict[str, Any] = dict(),
+    optimizer_args: Optional[dict[str, Any]] = None,
 ) -> AcquisitionOptimizer[Box | TaggedProductSearchSpace]:
     """
     Generate a gradient-based optimizer for :class:'Box' and :class:'TaggedProductSearchSpace'
@@ -267,19 +268,22 @@ def generate_continuous_optimizer(
             successes,
             fun_values,
             chosen_x,
+            nfev,
         ) = _perform_parallel_continuous_optimization(  # [num_optimization_runs, V]
             target_func,
             space,
             initial_points,
-            optimizer_args,
+            optimizer_args or {},
         )
 
         successful_optimization = tf.reduce_all(
             tf.reduce_any(successes, axis=0)
         )  # Check that at least one optimization was successful for each function
+        total_nfev = tf.reduce_max(nfev)  # acquisition function is evaluated in parallel
 
+        recovery_run = False
         if (
-            not successful_optimization
+            num_recovery_runs and not successful_optimization
         ):  # if all optimizations failed for a function then try again from random starts
             random_points = space.sample(num_recovery_runs)[:, None, :]  # [num_recovery_runs, 1, D]
             tiled_random_points = tf.tile(random_points, [1, V, 1])  # [num_recovery_runs, V, D]
@@ -288,8 +292,9 @@ def generate_continuous_optimizer(
                 recovery_successes,
                 recovery_fun_values,
                 recovery_chosen_x,
+                recovery_nfev,
             ) = _perform_parallel_continuous_optimization(
-                target_func, space, tiled_random_points, optimizer_args
+                target_func, space, tiled_random_points, optimizer_args or {}
             )
 
             successes = tf.concat(
@@ -305,6 +310,8 @@ def generate_continuous_optimizer(
             successful_optimization = tf.reduce_all(
                 tf.reduce_any(successes, axis=0)
             )  # Check that at least one optimization was successful for each function
+            total_nfev += tf.reduce_max(recovery_nfev)
+            recovery_run = True
 
         if not successful_optimization:  # return error if still failed
             raise FailedOptimizationError(
@@ -314,6 +321,31 @@ def generate_continuous_optimizer(
                     """
             )
 
+        summary_writer = logging.get_tensorboard_writer()
+        if summary_writer:
+            with summary_writer.as_default(step=logging.get_step_number()):
+
+                logging.scalar("spo_af_evaluations", total_nfev)
+                if recovery_run:
+                    logging.text(
+                        "spo_recovery_run",
+                        f"Acquisition function optimization failed after {num_optimization_runs} "
+                        f"optimization runs, requiring recovery runs",
+                    )
+
+                _target_func: AcquisitionFunction = target_func  # make mypy happy
+
+                def improvements() -> tf.Tensor:
+                    best_initial_values = tf.math.reduce_max(_target_func(initial_points), axis=0)
+                    best_values = tf.math.reduce_max(fun_values, axis=0)
+                    improve = best_values - tf.cast(best_initial_values, best_values.dtype)
+                    return improve[0] if V == 1 else improve
+
+                if V == 1:
+                    logging.scalar("spo_improvement_on_initial_samples", improvements)
+                else:
+                    logging.histogram("spo_improvement_on_initial_samples", improvements)
+
         best_run_ids = tf.math.argmax(fun_values, axis=0)  # [V]
         chosen_points = tf.gather(
             tf.transpose(chosen_x, [1, 0, 2]), best_run_ids, batch_dims=1
@@ -329,7 +361,7 @@ def _perform_parallel_continuous_optimization(
     space: SearchSpace,
     starting_points: TensorType,
     optimizer_args: dict[str, Any],
-) -> Tuple[TensorType, TensorType, TensorType]:
+) -> Tuple[TensorType, TensorType, TensorType, TensorType]:
     """
     A function to perform parallel optimization of our acquisition functions
     using Scipy. We perform L-BFGS-B starting from each of the locations contained
@@ -363,8 +395,8 @@ def _perform_parallel_continuous_optimization(
         `starting_points` controls the number of individual optimization runs
         for each of the V target functions.
     :param optimizer_args: Keyword arguments to pass to the Scipy optimizer.
-    :return: A tuple containing the failure status, the maximum value
-        and the maximiser found my each of our optimziations.
+    :return: A tuple containing the failure statuses, maximum values, maximisers and
+        number of evaluations for each of our optimizations.
     """
 
     tf_dtype = starting_points.dtype  # type for communication with Trieste
@@ -446,12 +478,16 @@ def _perform_parallel_continuous_optimization(
     vectorized_chosen_x = tf.constant(
         [result.x for result in final_vectorized_child_results], dtype=tf_dtype
     )  # [num_optimization_runs, D]
+    vectorized_nfev = tf.constant(
+        [result.nfev for result in final_vectorized_child_results], dtype=tf_dtype
+    )
 
     successes = tf.reshape(vectorized_successes, [-1, V])  # [num_optimization_runs, V]
     fun_values = tf.reshape(vectorized_fun_values, [-1, V])  # [num_optimization_runs, V]
     chosen_x = tf.reshape(vectorized_chosen_x, [-1, V, D])  # [num_optimization_runs, V, D]
+    nfev = tf.reshape(vectorized_nfev, [-1, V])  # [num_optimization_runs, V]
 
-    return (successes, fun_values, chosen_x)
+    return (successes, fun_values, chosen_x, nfev)
 
 
 class ScipyLbfgsBGreenlet(gr.greenlet):  # type: ignore[misc]
@@ -466,7 +502,7 @@ class ScipyLbfgsBGreenlet(gr.greenlet):  # type: ignore[misc]
         self,
         start: "np.ndarray[Any, Any]",
         bounds: spo.Bounds,
-        optimizer_args: dict[str, Any] = dict(),
+        optimizer_args: Optional[dict[str, Any]] = None,
     ) -> spo.OptimizeResult:
         cache_x = start + 1  # Any value different from `start`.
         cache_y: Optional["np.ndarray[Any, Any]"] = None
@@ -493,7 +529,7 @@ class ScipyLbfgsBGreenlet(gr.greenlet):  # type: ignore[misc]
             jac=lambda x: value_and_gradient(x)[1],
             method="l-bfgs-b",
             bounds=bounds,
-            **optimizer_args,
+            **(optimizer_args or {}),
         )
 
 
