@@ -22,8 +22,8 @@ to `False`.
 
 from . import config
 from .architectures import GaussianNetwork, KerasEnsemble, KerasEnsembleNetwork
-from .builders import build_vanilla_keras_ensemble
-from .interface import KerasPredictor
+from .builders import build_keras_ensemble
+from .interface import DeepEnsembleModel, KerasPredictor
 from .models import DeepEnsemble
-from .sampler import EnsembleTrajectorySampler
+from .sampler import DeepEnsembleTrajectorySampler
 from .utils import get_tensor_spec_from_data, negative_log_likelihood, sample_with_replacement
